/* This file is part of the KDE libraries
   Copyright (C) 2007-2008 Per Øyvind Karlsen <peroyvind@mandriva.org>

   Based on kbzip2filter:
   Copyright (C) 2000-2005 David Faure <faure@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "klzmafilter.h"

#include <config.h>
#define __STDC_LIMIT_MACROS 1 // Needed for UINT64_MAX in LZMA_VLI_VALUE_UNKNOWN
#include <sys/types.h>
#include <inttypes.h>

#if defined( HAVE_LZMA_SUPPORT )
extern "C" {
	#include <lzma.h>
}

#include <kdebug.h>

#include <qiodevice.h>


class KLzmaFilter::Private
{
public:
    Private()
    {
        memset(&zStream, 0, sizeof(zStream));
        mode = 0;
    }

    lzma_stream zStream;
    int mode;
};

KLzmaFilter::KLzmaFilter()
    :d(new Private)
{
}


KLzmaFilter::~KLzmaFilter()
{
    delete d;
}

void KLzmaFilter::init( int mode )
{
    lzma_ret result;
    d->zStream.next_in = 0;
    d->zStream.avail_in = 0;
    if ( mode == QIODevice::ReadOnly )
    {
        result = lzma_auto_decoder(&d->zStream, 0, 0);
    	//kDebug(7130) << "lzma_auto_decoder returned " << result;
    } else if ( mode == QIODevice::WriteOnly ) {
        // force old LZMA_Alone format since new lzma utils isn't stable yet..
        lzma_options_alone alone;
	alone.lzma = lzma_preset_lzma[6];
	alone.uncompressed_size = LZMA_VLI_VALUE_UNKNOWN;
        result = lzma_alone_encoder(&d->zStream, &alone);
    	//kDebug(7130) << "lzma_alone_encoder returned " << result;
    } else
        kWarning(7130) << "Unsupported mode " << mode << ". Only QIODevice::ReadOnly and QIODevice::WriteOnly supported";
    d->mode = mode;
}

int KLzmaFilter::mode() const
{
    return d->mode;
}

void KLzmaFilter::terminate()
{
    if ( d->mode == QIODevice::ReadOnly || d->mode == QIODevice::WriteOnly )
    {
        lzma_end(&d->zStream);
    } else
        kWarning(7130) << "Unsupported mode " << d->mode << ". Only QIODevice::ReadOnly and QIODevice::WriteOnly supported";
}


void KLzmaFilter::reset()
{
    kDebug(7130) << "KLzmaFilter::reset";
    // liblzma doesn't have a reset call...
    terminate();
    init( d->mode );
}

void KLzmaFilter::setOutBuffer( char * data, uint maxlen )
{
    d->zStream.avail_out = maxlen;
    d->zStream.next_out = (uint8_t *)data;
}

void KLzmaFilter::setInBuffer( const char *data, unsigned int size )
{
    d->zStream.avail_in = size;
    d->zStream.next_in = (uint8_t *)const_cast<char *>(data);
}

int KLzmaFilter::inBufferAvailable() const
{
    return d->zStream.avail_in;
}

int KLzmaFilter::outBufferAvailable() const
{
    return d->zStream.avail_out;
}

KLzmaFilter::Result KLzmaFilter::uncompress()
{
    //kDebug(7130) << "Calling lzma_code with avail_in=" << inBufferAvailable() << " avail_out =" << outBufferAvailable();
    lzma_ret result = lzma_code(&d->zStream, LZMA_RUN);
    if ( result != LZMA_OK )
    {
        kDebug(7130) << "lzma_code returned " << result;
        kDebug(7130) << "KLzmaFilter::uncompress " << ( result == LZMA_STREAM_END ? KFilterBase::End : KFilterBase::Error );
    }

    switch (result) {
        case LZMA_OK:
                return KFilterBase::Ok;
        case LZMA_STREAM_END:
                return KFilterBase::End;
        default:
                return KFilterBase::Error;
    }
}

KLzmaFilter::Result KLzmaFilter::compress( bool finish )
{
    //kDebug(7130) << "Calling lzma_code with avail_in=" << inBufferAvailable() << " avail_out=" << outBufferAvailable();
    lzma_ret result = lzma_code(&d->zStream, finish ? LZMA_FINISH : LZMA_RUN );

    switch (result) {
        case LZMA_OK:
                return KFilterBase::Ok;
                break;
        case LZMA_STREAM_END:
                kDebug(7130) << "  lzma_code returned " << result;
                return KFilterBase::End;
		break;
        default:
                kDebug(7130) << "  lzma_code returned " << result;
                return KFilterBase::Error;
                break;
    }
}

#endif  /* HAVE_LZMA_SUPPORT */
